// Copyright 2020 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by client-gen. DO NOT EDIT.

package v1alpha1

import (
	"context"
	"time"

	v1alpha1 "github.com/google/kf/v2/pkg/apis/kf/v1alpha1"
	scheme "github.com/google/kf/v2/pkg/client/kf/clientset/versioned/scheme"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// ServiceBrokersGetter has a method to return a ServiceBrokerInterface.
// A group's client should implement this interface.
type ServiceBrokersGetter interface {
	ServiceBrokers(namespace string) ServiceBrokerInterface
}

// ServiceBrokerInterface has methods to work with ServiceBroker resources.
type ServiceBrokerInterface interface {
	Create(ctx context.Context, serviceBroker *v1alpha1.ServiceBroker, opts v1.CreateOptions) (*v1alpha1.ServiceBroker, error)
	Update(ctx context.Context, serviceBroker *v1alpha1.ServiceBroker, opts v1.UpdateOptions) (*v1alpha1.ServiceBroker, error)
	UpdateStatus(ctx context.Context, serviceBroker *v1alpha1.ServiceBroker, opts v1.UpdateOptions) (*v1alpha1.ServiceBroker, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*v1alpha1.ServiceBroker, error)
	List(ctx context.Context, opts v1.ListOptions) (*v1alpha1.ServiceBrokerList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.ServiceBroker, err error)
	ServiceBrokerExpansion
}

// serviceBrokers implements ServiceBrokerInterface
type serviceBrokers struct {
	client rest.Interface
	ns     string
}

// newServiceBrokers returns a ServiceBrokers
func newServiceBrokers(c *KfV1alpha1Client, namespace string) *serviceBrokers {
	return &serviceBrokers{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the serviceBroker, and returns the corresponding serviceBroker object, and an error if there is any.
func (c *serviceBrokers) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1alpha1.ServiceBroker, err error) {
	result = &v1alpha1.ServiceBroker{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("servicebrokers").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of ServiceBrokers that match those selectors.
func (c *serviceBrokers) List(ctx context.Context, opts v1.ListOptions) (result *v1alpha1.ServiceBrokerList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1alpha1.ServiceBrokerList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("servicebrokers").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested serviceBrokers.
func (c *serviceBrokers) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("servicebrokers").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a serviceBroker and creates it.  Returns the server's representation of the serviceBroker, and an error, if there is any.
func (c *serviceBrokers) Create(ctx context.Context, serviceBroker *v1alpha1.ServiceBroker, opts v1.CreateOptions) (result *v1alpha1.ServiceBroker, err error) {
	result = &v1alpha1.ServiceBroker{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("servicebrokers").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(serviceBroker).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a serviceBroker and updates it. Returns the server's representation of the serviceBroker, and an error, if there is any.
func (c *serviceBrokers) Update(ctx context.Context, serviceBroker *v1alpha1.ServiceBroker, opts v1.UpdateOptions) (result *v1alpha1.ServiceBroker, err error) {
	result = &v1alpha1.ServiceBroker{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("servicebrokers").
		Name(serviceBroker.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(serviceBroker).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *serviceBrokers) UpdateStatus(ctx context.Context, serviceBroker *v1alpha1.ServiceBroker, opts v1.UpdateOptions) (result *v1alpha1.ServiceBroker, err error) {
	result = &v1alpha1.ServiceBroker{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("servicebrokers").
		Name(serviceBroker.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(serviceBroker).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the serviceBroker and deletes it. Returns an error if one occurs.
func (c *serviceBrokers) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("servicebrokers").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *serviceBrokers) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("servicebrokers").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched serviceBroker.
func (c *serviceBrokers) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.ServiceBroker, err error) {
	result = &v1alpha1.ServiceBroker{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("servicebrokers").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
