// Copyright 2020 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by injection-gen. DO NOT EDIT.

package filtered

import (
	context "context"

	versioned "github.com/google/kf/v2/pkg/client/kube-aggregator/clientset/versioned"
	v1 "github.com/google/kf/v2/pkg/client/kube-aggregator/informers/externalversions/apiregistration/v1"
	client "github.com/google/kf/v2/pkg/client/kube-aggregator/injection/client"
	filtered "github.com/google/kf/v2/pkg/client/kube-aggregator/injection/informers/factory/filtered"
	apiregistrationv1 "github.com/google/kf/v2/pkg/client/kube-aggregator/listers/apiregistration/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	labels "k8s.io/apimachinery/pkg/labels"
	cache "k8s.io/client-go/tools/cache"
	apisapiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
	controller "knative.dev/pkg/controller"
	injection "knative.dev/pkg/injection"
	logging "knative.dev/pkg/logging"
)

func init() {
	injection.Default.RegisterFilteredInformers(withInformer)
	injection.Dynamic.RegisterDynamicInformer(withDynamicInformer)
}

// Key is used for associating the Informer inside the context.Context.
type Key struct {
	Selector string
}

func withInformer(ctx context.Context) (context.Context, []controller.Informer) {
	untyped := ctx.Value(filtered.LabelKey{})
	if untyped == nil {
		logging.FromContext(ctx).Panic(
			"Unable to fetch labelkey from context.")
	}
	labelSelectors := untyped.([]string)
	infs := []controller.Informer{}
	for _, selector := range labelSelectors {
		f := filtered.Get(ctx, selector)
		inf := f.Apiregistration().V1().APIServices()
		ctx = context.WithValue(ctx, Key{Selector: selector}, inf)
		infs = append(infs, inf.Informer())
	}
	return ctx, infs
}

func withDynamicInformer(ctx context.Context) context.Context {
	untyped := ctx.Value(filtered.LabelKey{})
	if untyped == nil {
		logging.FromContext(ctx).Panic(
			"Unable to fetch labelkey from context.")
	}
	labelSelectors := untyped.([]string)
	for _, selector := range labelSelectors {
		inf := &wrapper{client: client.Get(ctx), selector: selector}
		ctx = context.WithValue(ctx, Key{Selector: selector}, inf)
	}
	return ctx
}

// Get extracts the typed informer from the context.
func Get(ctx context.Context, selector string) v1.APIServiceInformer {
	untyped := ctx.Value(Key{Selector: selector})
	if untyped == nil {
		logging.FromContext(ctx).Panicf(
			"Unable to fetch github.com/google/kf/v2/pkg/client/kube-aggregator/informers/externalversions/apiregistration/v1.APIServiceInformer with selector %s from context.", selector)
	}
	return untyped.(v1.APIServiceInformer)
}

type wrapper struct {
	client versioned.Interface

	selector string
}

var _ v1.APIServiceInformer = (*wrapper)(nil)
var _ apiregistrationv1.APIServiceLister = (*wrapper)(nil)

func (w *wrapper) Informer() cache.SharedIndexInformer {
	return cache.NewSharedIndexInformer(nil, &apisapiregistrationv1.APIService{}, 0, nil)
}

func (w *wrapper) Lister() apiregistrationv1.APIServiceLister {
	return w
}

func (w *wrapper) List(selector labels.Selector) (ret []*apisapiregistrationv1.APIService, err error) {
	reqs, err := labels.ParseToRequirements(w.selector)
	if err != nil {
		return nil, err
	}
	selector = selector.Add(reqs...)
	lo, err := w.client.ApiregistrationV1().APIServices().List(context.TODO(), metav1.ListOptions{
		LabelSelector: selector.String(),
		// TODO(mattmoor): Incorporate resourceVersion bounds based on staleness criteria.
	})
	if err != nil {
		return nil, err
	}
	for idx := range lo.Items {
		ret = append(ret, &lo.Items[idx])
	}
	return ret, nil
}

func (w *wrapper) Get(name string) (*apisapiregistrationv1.APIService, error) {
	// TODO(mattmoor): Check that the fetched object matches the selector.
	return w.client.ApiregistrationV1().APIServices().Get(context.TODO(), name, metav1.GetOptions{
		// TODO(mattmoor): Incorporate resourceVersion bounds based on staleness criteria.
	})
}
